import arviz as az
import matplotlib.pyplot as plt
import numpy as np
import seaborn as sb
import scipy.stats as ss
import pickle
import pandas as pd

pd.set_option('display.max_columns', None)

def CalcMAPEst(samples, a = 0.05, n_points = 2000):
    """
        Calculatte a MAP estimate from a KDE graph on [Lp, Up]
        Lp and Up are 100*a/2 and 100(1-a/2) percentile points of samples
    """
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  # import numpy as np
    coord = np.linspace(Lp, Up, n_points) 
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  # import scipy.stats as ss
    map_idx = np.argmax(est_pdf) 
    MAP_Est = coord[map_idx]       
    return MAP_Est, est_pdf[map_idx]

fout = open('Output_Anal.txt', 'w')

with open('noise.pkl', 'rb') as f:
    Data_Noise = pickle.load(f)
with open('signal.pkl', 'rb') as f:
    Data_Signal = pickle.load(f)
K = len(Data_Noise)

fout.write('\nData for Noise:\n')
fout.write('    {}\n'.format(Data_Noise))
fout.write('\nData for Signal:\n')
fout.write('    {}'.format(Data_Signal))
fout.write('\n')

with open('infrnc_d.pkl', 'rb') as f:
    infrnc_d = pickle.load(f)
with open('d_frame.pkl', 'rb') as f:
    d_frame = pickle.load(f)

az.plot_trace(infrnc_d)
plt.tight_layout()
plt.savefig('FigTrace.png')
plt.show()

mu = d_frame['mu']   
sigma = d_frame['sigma']   
mu_mean = mu.mean()
mu_MAP = CalcMAPEst(mu)[0]
mu_q025, mu_med, mu_q975 = np.percentile(mu, [2.5, 50, 97.5]) 
sgm_mean = sigma.mean()
sgm_MAP = CalcMAPEst(sigma)[0]
sgm_q025, sgm_med, sgm_q975 = np.percentile(sigma, [2.5, 50, 97.5])  

fout.write('\nmu:\n')
fout.write('    MAP est. = {0:.3f}\n'.format(mu_MAP))
fout.write('    med.     = {0:.3f}\n'.format(mu_med))
fout.write('    mean     = {0:.3f}\n'.format(mu_mean))
fout.write('    95% CI   = [{0:.3f}, {1:.3f}]\n'.format(mu_q025, mu_q975))

fout.write('\nsigma:\n')
fout.write('    MAP est. = {0:.3f}\n'.format(sgm_MAP))
fout.write('    med.     = {0:.3f}\n'.format(sgm_med))
fout.write('    mean     = {0:.3f}\n'.format(sgm_mean))
fout.write('    95% CI   = [{0:.3f}, {1:.3f}]\n'.format(sgm_q025, sgm_q975))

sb.kdeplot(mu)
plt.title((r'$\mu$') + (r'($\Delta$') + 'm) = ' +
           '{0:<.3f}(mode)'.format(mu_MAP),
          fontsize = 18)
plt.yticks([])
plt.ylabel('Density', fontsize = 14)
plt.xlabel('$\mu$', fontsize = 16)
plt.tight_layout()
plt.savefig('Figure_mu.png')
plt.show()

sb.kdeplot(sigma)
plt.title("$\sigma$ = {0:<.3f}(mode) ".format(sgm_MAP), fontsize = 18)
plt.yticks([])
plt.ylabel('Density', fontsize = 14)
plt.xlabel('$\sigma$', fontsize = 18)
plt.tight_layout()
plt.savefig('Figure_sgm.png')
plt.show()

c = [0]*(K-1)
for k in range(K-1):
    c[k] = d_frame[f'c.{k+1}']
c = np.array(c).T
print('c =\n', c)

z_values = np.arange(-4.0, 4.001, 0.1)
x_values = 1.0 - ss.norm(0.0, 1.0).cdf(z_values)
y_values = 1.0 - ss.norm(mu_MAP, sgm_MAP).cdf(z_values)

plt.figure(figsize = (5,5))
plt.plot(x_values, y_values, 'b-')
plt.title("ROC-curve", fontsize = 18)
plt.xlabel("P(False Alarm)", fontsize = 14)
plt.ylabel("P(Hit)", fontsize = 14)

cum_freq_n = []
cum_freq_s = []
for k in range(K):
    cum_freq_n.append(Data_Noise[K - 1 - k])
    cum_freq_s.append(Data_Signal[K - 1 - k])

for k in range(1, K):
    cum_freq_n[k] += cum_freq_n[k-1]
    cum_freq_s[k] += cum_freq_s[k-1]

x_fa = []
y_ht = []
for k in range(K-1):
    x_fa.append(cum_freq_n[k] / cum_freq_n[K-1])
    y_ht.append(cum_freq_s[k] / cum_freq_s[K-1])
plt.plot(x_fa, y_ht, 'ro', label = 'Data')

print('\nI am calculating...')

c_means = np.zeros(K-1)
c_meds = np.zeros(K-1)
c_MAPs = np.zeros(K-1)
c_q025s = np.zeros(K-1)
c_q975s = np.zeros(K-1)
for k in range(K-1):
    c_means[k] = c.T[k].mean()
    c_MAPs[k] = CalcMAPEst(c.T[k])[0]
    c_q025s[k], c_meds[k], c_q975s[k] = np.percentile(c.T[k], [2.5, 50, 97.5])

fout.write('\n')
for k in range(K-1):
    fout.write('\nC{}:\n'.format(k+1))
    fout.write('    MAP est. = {0:.3f}\n'.format(c_MAPs[k]))
    fout.write('    median   = {0:.3f}\n'.format(c_meds[k]))
    fout.write('    mean     = {0:.3f}\n'.format(c_means[k]))
    fout.write('    95% CI   = [{0:.3f}, {1:.3f}]\n'.format(
                    c_q025s[k], c_q975s[k]))
    
x_fa = []
y_ht = []

for k in range(K-1):
    x_fa.append(1.0 - ss.norm().cdf(c_MAPs[k]))  
    y_ht.append(1.0 - ss.norm(mu_MAP, sgm_MAP).cdf(c_MAPs[k]))  
plt.plot(x_fa, y_ht, 'gs', label = 'Prediction')

plt.legend(fontsize = 12)
plt.tight_layout()
plt.savefig('Figure_ROC.png')
plt.show()

for k in range(K-1):
    sb.kdeplot(c.T[k], label = '$C_{0}$: mode = {1:.3f}'.format(k+1, c_MAPs[k]))
title_str = "Criterions" 
plt.title(title_str, fontsize = 18)
plt.yticks([])
plt.ylabel('Density', fontsize = 14)
plt.xlabel('C', fontsize = 14)
plt.legend(fontsize = 12)
plt.tight_layout()
plt.savefig('Figure_Cs.png')
plt.show()

fout.close()
print('Output_Anal.txt was saved.')
